//////////////////////////////////////////////////////////////////////
//
// ApnCamData.cpp:  Implementation of the CApnCamData class.
//
// Copyright (c) 2003-2007 Apogee Instruments, Inc.
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this file,
// You can obtain one at http://mozilla.org/MPL/2.0/.
//////////////////////////////////////////////////////////////////////


#include "ApnCamData.h"

#include "camMetaData.h"
#include "patterns.h"
#include "apgHelper.h" 
#include "helpers.h"

#include <iostream>
#include <fstream>

//////////////////////////// 
// CTOR 
CApnCamData::CApnCamData()
{
    Clear();
}


//////////////////////////// 
// CTOR 
CApnCamData::CApnCamData(const CamCfg::APN_CAMERA_METADATA & meta,
        const CamCfg::APN_VPATTERN_FILE & vert,
        const CamCfg::APN_HPATTERN_FILE & clampNorm,
        const CamCfg::APN_HPATTERN_FILE & skipNorm,
        const CamCfg::APN_HPATTERN_FILE & roiNorm,
        const CamCfg::APN_HPATTERN_FILE & clampFast,
        const CamCfg::APN_HPATTERN_FILE & skipFast,
        const CamCfg::APN_HPATTERN_FILE & roiFast,
        const CamCfg::APN_VPATTERN_FILE & vertVideo,
        const CamCfg::APN_HPATTERN_FILE & clampVideo,
        const CamCfg::APN_HPATTERN_FILE & skipVideo,
        const CamCfg::APN_HPATTERN_FILE & roiVideo) :
        m_MetaData( meta ),
        m_VerticalPattern( vert ),
        m_ClampPatternNormal( clampNorm ),
        m_SkipPatternNormal( skipNorm ),
        m_RoiPatternNormal( roiNorm ),
        m_ClampPatternFast( clampFast ),
        m_SkipPatternFast( skipFast ),
        m_RoiPatternFast( roiFast ),
        m_VerticalPatternVideo( vertVideo ),
        m_ClampPatternVideo( clampVideo ),
        m_SkipPatternVideo( skipVideo ),
        m_RoiPatternVideo( roiVideo )
{
        
}

//////////////////////////// 
// COPY       CTOR 
CApnCamData::CApnCamData( const CApnCamData &rhs ) :
    m_MetaData( rhs.m_MetaData ),
        m_VerticalPattern( rhs.m_VerticalPattern ),
        m_ClampPatternNormal( rhs.m_ClampPatternNormal ),
        m_SkipPatternNormal( rhs.m_SkipPatternNormal ),
        m_RoiPatternNormal( rhs.m_RoiPatternNormal ),
        m_ClampPatternFast( rhs.m_ClampPatternFast ),
        m_SkipPatternFast( rhs.m_SkipPatternFast ),
        m_RoiPatternFast( rhs.m_RoiPatternFast ),
        m_VerticalPatternVideo( rhs.m_VerticalPatternVideo ),
        m_ClampPatternVideo( rhs.m_ClampPatternVideo ),
        m_SkipPatternVideo( rhs.m_SkipPatternVideo ),
        m_RoiPatternVideo( rhs.m_RoiPatternVideo )
{
}

//////////////////////////// 
// DTOR 
CApnCamData::~CApnCamData()
{
    Clear();
}

//////////////////////////// 
//      ASSIGNMENT    OPERATOR
CApnCamData& CApnCamData::operator=(CApnCamData const&d)
{
     // Gracefully handle self assignment
    // from http://www.parashift.com/c++-faq-lite/assignment-operators.html
    if (this != &d) 
    {  
        this->m_ClampPatternFast = d.m_ClampPatternFast;
        this->m_ClampPatternNormal = d.m_ClampPatternNormal;
        this->m_ClampPatternVideo = d.m_ClampPatternVideo;
        this->m_MetaData = d.m_MetaData;
        this->m_RoiPatternFast = d.m_RoiPatternFast;
        this->m_RoiPatternNormal = d.m_RoiPatternNormal;
        this->m_RoiPatternVideo = d.m_RoiPatternVideo;
        this->m_SkipPatternFast = d.m_SkipPatternFast;
        this->m_SkipPatternNormal = d.m_SkipPatternNormal;
        this->m_SkipPatternVideo = d.m_SkipPatternVideo;
        this->m_VerticalPattern = d.m_VerticalPattern;
        this->m_VerticalPatternVideo = d.m_VerticalPatternVideo;
    }
   return *this;
}

//////////////////////////// 
// SET 
void CApnCamData::Set(const uint16_t CamId)
{
    camMetaData::getMetaPtr fxPtr = 0;

    if( !camMetaData::getMetaFxPtr( CamId, fxPtr ) )
    {
         std::string errStr = "Invalid camera id " + 
             help::uShort2Str(CamId);
        apgHelper::throwRuntimeException( __FILE__, errStr, __LINE__,
            Apg::ErrorType_Configuration );
    }
    
    //fetch the sensor data
    //can throw if the ad types are invalid
    try
    {
        fxPtr( m_MetaData );
    }
    catch( std::exception & e)
    {
        std::string errStr( e.what() );
        errStr += " exception thrown for camera id = " + help::uShort2Str( CamId );
        apgHelper::throwRuntimeException( __FILE__, errStr, 
            __LINE__, Apg::ErrorType_Configuration );
    }

    //use the sensor data to
    //get the pattern data
    LoadVertical( m_MetaData.VerticalPattern, m_VerticalPattern );
    
    //normal clamp
    LoadHorizontal(  m_MetaData.ClampPatternNormal, m_ClampPatternNormal );
 
    //normal skip
    LoadHorizontal(  m_MetaData.SkipPatternNormal, m_SkipPatternNormal );

    //normal roi
    LoadHorizontal(  m_MetaData.RoiPatternNormal, m_RoiPatternNormal );

    //fast clamp
    LoadHorizontal(  m_MetaData.ClampPatternFast, m_ClampPatternFast );

    //Fast skip
    LoadHorizontal(  m_MetaData.SkipPatternFast, m_SkipPatternFast );

    //Fast roi
    LoadHorizontal(  m_MetaData.RoiPatternFast, m_RoiPatternFast );

    //veritcal video
    LoadVertical( m_MetaData.VerticalPatternVideo, m_VerticalPatternVideo );
    
    //normal clamp
    LoadHorizontal(  m_MetaData.ClampPatternVideo, m_ClampPatternVideo );
 
    //normal skip
    LoadHorizontal(  m_MetaData.SkipPatternVideo, m_SkipPatternVideo );

    //normal roi
    LoadHorizontal(  m_MetaData.RoiPatternVideo, m_RoiPatternVideo );
   
}

//////////////////////////// 
// LOAD       VERTICAL
void CApnCamData::LoadVertical( const std::string & name, 
            CamCfg::APN_VPATTERN_FILE & vpattern )
{
     patterns::getVPatternPtr vFx = 0;
    if( !patterns::GetVPatternFx( name, vFx) )
    {
        std::string errStr = "Invalid vertical pattern " + name;
        apgHelper::throwRuntimeException( __FILE__, errStr, __LINE__,
            Apg::ErrorType_Configuration );
    }
        
    vFx( vpattern );
}

//////////////////////////// 
// LOAD       HORIZONTAL
void CApnCamData::LoadHorizontal( const std::string & name, 
            CamCfg::APN_HPATTERN_FILE & hpattern )
{
    patterns::getHPatternPtr hFx = 0;

    if( !patterns::GetHPatternFx( name, hFx) )
    {
        std::string errStr = "Invalid horizontal pattern " + name;
        apgHelper::throwRuntimeException( __FILE__, errStr, __LINE__,
            Apg::ErrorType_Configuration );
    }

    hFx( hpattern );
}

//////////////////////////// 
// CLEAR
void CApnCamData::Clear()
{
    CamCfg::Clear( m_MetaData );
    CamCfg::Clear( m_VerticalPattern );
    CamCfg::Clear( m_ClampPatternNormal );
    CamCfg::Clear( m_SkipPatternNormal );
    CamCfg::Clear( m_RoiPatternNormal );
    CamCfg::Clear( m_ClampPatternFast);
    CamCfg::Clear( m_SkipPatternFast );
    CamCfg::Clear( m_RoiPatternFast );
    CamCfg::Clear( m_VerticalPatternVideo );
    CamCfg::Clear( m_ClampPatternVideo );
    CamCfg::Clear( m_SkipPatternVideo );
    CamCfg::Clear( m_RoiPatternVideo );
}

void CApnCamData::Write2File( const std::string & fname )
{
    std::ofstream f( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "cfg data" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteMeta(  fname );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Vertical" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteVPattern( fname, m_VerticalPattern );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Clamp Normal" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteHPattern( fname, m_ClampPatternNormal );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Skip Normal" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteHPattern( fname, m_SkipPatternNormal );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Roi Normal" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteHPattern( fname, m_RoiPatternNormal );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Clamp Fast" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteHPattern( fname, m_ClampPatternFast );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Skip Fast" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteHPattern( fname, m_SkipPatternFast );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Roi Fast" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteHPattern( fname, m_RoiPatternFast );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Vertical Video" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteVPattern( fname, m_VerticalPatternVideo );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Clamp Video" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteHPattern( fname, m_ClampPatternVideo );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Skip Video" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteHPattern( fname, m_SkipPatternVideo );

    f.open( fname.c_str(), std::ios::out | std::ios::app );
    f << "---------------------------------------" << std::endl;
    f << "Roi Video" << std::endl;
    f << "---------------------------------------" << std::endl;
    f.close();

    WriteHPattern( fname, m_RoiPatternVideo );
}

void CApnCamData::WriteMeta( const std::string & fname )
{
    std::ofstream f( fname.c_str(), std::ios::out | std::ios::app );

    f << "Sensor: " << m_MetaData.Sensor << std::endl;
    f << "CameraId: " << m_MetaData.CameraId << std::endl;
    f << "CameraLine: " << m_MetaData.CameraLine << std::endl;
    f << "CameraModel: " << m_MetaData.CameraModel << std::endl;
    f << "InterlineCCD: " << m_MetaData.InterlineCCD << std::endl;
    f << "SupportsSerialA: " << m_MetaData.SupportsSerialA << std::endl;
    f << "SupportsSerialB: " << m_MetaData.SupportsSerialB << std::endl;
    f << "SensorTypeCCD: " << m_MetaData.SensorTypeCCD << std::endl;
    f << "TotalColumns: " << m_MetaData.TotalColumns << std::endl;
    f << "ImagingColumns: " << m_MetaData.ImagingColumns << std::endl;
    f << "ClampColumns: " << m_MetaData.ClampColumns << std::endl;
    f << "PreRoiSkipColumns: " << m_MetaData.PreRoiSkipColumns << std::endl;
    f << "PostRoiSkipColumns: " << m_MetaData.PostRoiSkipColumns << std::endl;
    f << "OverscanColumns: " << m_MetaData.OverscanColumns << std::endl;
    f << "TotalRows: " << m_MetaData.TotalRows << std::endl;
    f << "ImagingRows: " << m_MetaData.ImagingRows << std::endl;
    f << "UnderscanRows: " << m_MetaData.UnderscanRows << std::endl;
    f << "OverscanRows: " << m_MetaData.OverscanRows << std::endl;
    f << "VFlushBinning: " << m_MetaData.VFlushBinning << std::endl;
    f << "EnableSingleRowOffset: " << m_MetaData.EnableSingleRowOffset << std::endl;
    f << "RowOffsetBinning: " << m_MetaData.RowOffsetBinning << std::endl;
    f << "HFlushDisable: " << m_MetaData.HFlushDisable << std::endl;
    f << "ShutterCloseDelay: " << m_MetaData.ShutterCloseDelay << std::endl;
    f << "PixelSizeX: " << m_MetaData.PixelSizeX << std::endl;
    f << "PixelSizeY: " << m_MetaData.PixelSizeY << std::endl;
    f << "Color: " << m_MetaData.Color << std::endl;
    f << "ReportedGainSixteenBit: " << m_MetaData.ReportedGainSixteenBit << std::endl;
    f << "MinSuggestedExpTime: " << m_MetaData.MinSuggestedExpTime << std::endl;
    f << "CoolingSupported: " << m_MetaData.CoolingSupported << std::endl;
    f << "RegulatedCoolingSupported: " << m_MetaData.RegulatedCoolingSupported << std::endl;
    f << "TempSetPoint: " << m_MetaData.TempSetPoint << std::endl;
    f << "TempRampRateOne: " << m_MetaData.TempRampRateOne << std::endl;
    f << "TempRampRateTwo: " << m_MetaData.TempRampRateTwo << std::endl;
    f << "TempBackoffPoint: " << m_MetaData.TempBackoffPoint << std::endl;
    f << "PrimaryADType: " << m_MetaData.PrimaryADType << std::endl;
    f << "AlternativeADType: " << m_MetaData.AlternativeADType << std::endl;
    f << "PrimaryADLatency: " << m_MetaData.PrimaryADLatency << std::endl;
    f << "AlternativeADLatency: " << m_MetaData.AlternativeADLatency << std::endl;
    f << "IRPreflashTime: " << m_MetaData.IRPreflashTime << std::endl;
    f << "AdCfg: " << m_MetaData.AdCfg << std::endl;
    f << "DefaultGainLeft: " << m_MetaData.DefaultGainLeft << std::endl;
    f << "DefaultOffsetLeft: " << m_MetaData.DefaultOffsetLeft << std::endl;
    f << "DefaultGainRight: " << m_MetaData.DefaultGainRight << std::endl;
    f << "DefaultOffsetRight: " << m_MetaData.DefaultOffsetRight << std::endl;
    f << "DefaultRVoltage: " << m_MetaData.DefaultRVoltage << std::endl;
    f << "DefaultDataReduction: " << m_MetaData.DefaultDataReduction << std::endl;
    f << "VideoSubSample: " << m_MetaData.VideoSubSample << std::endl;
    f << "AmpCutoffDisable: " << m_MetaData.AmpCutoffDisable << std::endl;
    f << "NumAdOutputs: " << m_MetaData.NumAdOutputs << std::endl;
    f << "VerticalPattern: " << m_MetaData.VerticalPattern << std::endl;
    f << "SkipPatternNormal: " << m_MetaData.SkipPatternNormal << std::endl;
    f << "ClampPatternNormal: " << m_MetaData.ClampPatternNormal << std::endl;
    f << "RoiPatternNormal: " << m_MetaData.RoiPatternNormal << std::endl;
    f << "SkipPatternFast: " << m_MetaData.SkipPatternFast << std::endl;
    f << "ClampPatternFast: " << m_MetaData.ClampPatternFast << std::endl;
    f << "RoiPatternFast: " << m_MetaData.RoiPatternFast << std::endl;
    f << "VerticalPatternVideo: " << m_MetaData.VerticalPatternVideo << std::endl;
    f << "SkipPatternVideo: " << m_MetaData.SkipPatternVideo << std::endl;
    f << "ClampPatternVideo: " << m_MetaData.ClampPatternVideo << std::endl;
    f << "RoiPatternVideo: " << m_MetaData.RoiPatternVideo << std::endl;

    f.close();
}


void CApnCamData::WriteHPattern( const std::string & fname,
                                const CamCfg::APN_HPATTERN_FILE & horiztonal)
{
     std::ofstream f( fname.c_str(), std::ios::out | std::ios::app );

    f << "Mask: " << horiztonal.Mask << std::endl;

    f << "Ref: ";

    std::vector<uint16_t>::const_iterator iter;

    for( iter = horiztonal.RefPatternData.begin(); 
        iter != horiztonal.RefPatternData.end(); ++iter )
    {
          f << (*iter) << " ";
    }

    f << std::endl;

    f << "Sig: ";

    for( iter = horiztonal.SigPatternData.begin(); 
        iter != horiztonal.SigPatternData.end(); ++iter )
    {
          f << (*iter) << " ";
    }

    f << std::endl;

    int c = 0;
    std::vector< std::vector<uint16_t> >::const_iterator jj;
    for( jj = horiztonal.BinPatternData.begin(); 
        jj != horiztonal.BinPatternData.end(); ++jj, ++c )
    {
        f << "Bin " << c << ": ";
        std::vector<uint16_t>::const_iterator ii;
        for( ii = (*jj).begin(); ii != (*jj).end(); ++ii )
        {
            f << (*ii) << " ";
        }

        f << std::endl;
    }

    f << std::endl;

    f.close();

}


void CApnCamData::WriteVPattern( const std::string & fname, 
                                const CamCfg::APN_VPATTERN_FILE & vert )
{
    std::ofstream f( fname.c_str(), std::ios::out | std::ios::app );

     f << "Mask: " << vert.Mask << std::endl;

    f << "Pattern: ";

    std::vector<uint16_t>::const_iterator iter;

    for( iter = vert.PatternData.begin(); iter != vert.PatternData.end(); ++iter )
    {
          f << (*iter) << " ";
    }

    f << std::endl;
    f.close();
}











